package com.qmth.distributed.print.api;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.service.ActivitiService;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: activiti demo api
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/7/12
 */
@Api(tags = "工作流Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.basic}/activiti")
@Aac(strict = BOOL.FALSE, auth = BOOL.FALSE)
public class ActivitiController {

    @Resource
    private ActivitiService activitiService;

    @ApiOperation(value = "注册流程")
    @RequestMapping(value = "/createDeployment", method = RequestMethod.POST)
    public Result createDeployment(@RequestParam(value = "processFileName") String processFileName) {
        activitiService.createDeployment(processFileName);
        return ResultUtil.ok();
    }

    @ApiOperation(value = "上传流程")
    @RequestMapping(value = "/updateDeployment", method = RequestMethod.POST)
    public Result uploadDeployment(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file) throws IOException {
        activitiService.uploadDeployment(file);
        return ResultUtil.ok();
    }

    @ApiOperation(value = "启动流程")
    @RequestMapping(value = "/startActivity", method = RequestMethod.POST)
    public Result startActivity(@RequestParam(value = "processKey") String processKey) {
        Map<String, Object> map = new HashMap<>();
        map.put("userId", 1L);
        map.put("approveId", 2L);
        map.put("assigneeId", 3L);
        activitiService.startActivity(processKey, map);
        return ResultUtil.ok();
    }

    @ApiOperation(value = "获取待办")
    @RequestMapping(value = "/getTaskList", method = RequestMethod.POST)
    public Result getTaskList() {
        activitiService.getTaskList();
        return ResultUtil.ok();
    }

    @ApiOperation(value = "提交")
    @RequestMapping(value = "/complete", method = RequestMethod.POST)
    public Result complete(@RequestParam(value = "taskId") String taskId) {
        activitiService.complete(taskId);
        return ResultUtil.ok();
    }

    @ApiOperation(value = "根据流程id直接结束流程")
    @RequestMapping(value = "/deleteProcessInstance", method = RequestMethod.POST)
    public Result deleteProcessInstance(@RequestParam(value = "flowId") String flowId) {
        activitiService.deleteProcessInstance(flowId);
        return ResultUtil.ok();
    }
}