package com.qmth.teachcloud.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicSchool;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.OrgCenterTypeEnum;
import com.qmth.teachcloud.common.service.BasicSchoolService;
import com.qmth.teachcloud.common.service.CallApiOrgCenterService;
import com.qmth.teachcloud.common.service.OrgCenterDataDisposeService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 机构数据中心数据处理服务实现类
 * @Author: CaoZixuan
 * @Date: 2021-04-02
 */
@Service
public class OrgCenterDataDisposeServiceImpl implements OrgCenterDataDisposeService {
    @Resource
    private CallApiOrgCenterService callApiOrgCenterService;
    @Resource
    private BasicSchoolService basicSchoolService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateSchoolInfo() throws IOException {
        List<Map> result = callApiOrgCenterService.callOrgInfo();
        if (Objects.isNull(result) || result.size() == 0) {
            return;
        }
        // 删除原有学校
        List<BasicSchool> basicSchoolList = basicSchoolService.list();
        List<Long> oldIdList = basicSchoolList.stream().map(BasicSchool::getId).collect(Collectors.toList());
        List<Long> newIdList = result.stream().map(e -> SystemConstant.convertIdToLong(String.valueOf(e.get("id")))).collect(Collectors.toList());
        List<Long> needDisable = new ArrayList<>();
        List<Long> needDelete = new ArrayList<>();
        for (Long aLong : oldIdList) {
            if (!newIdList.contains(aLong)) {
                needDisable.add(aLong);
            } else {
                needDelete.add(aLong);
            }
        }
        if (needDisable.size() > 0) {
            basicSchoolService.update(new UpdateWrapper<BasicSchool>().lambda().set(BasicSchool::getEnable, false).in(BasicSchool::getId, needDisable));
        }
        if (needDelete.size() > 0) {
            basicSchoolService.remove(new QueryWrapper<BasicSchool>().lambda().in(BasicSchool::getId, needDelete));
        }

        for (Map map : result) {
            Long id = SystemConstant.convertIdToLong(String.valueOf(map.get("id")));
            String code = String.valueOf(map.get("code"));
            String name = String.valueOf(map.get("name"));
            String logo = String.valueOf(map.get("logo"));
            String accessKey = String.valueOf(map.get("accessKey"));
            String accessSecret = String.valueOf(map.get("accessSecret"));
            Map typeMap = JSONObject.parseObject(String.valueOf(map.get("type")));
            String typeCode = String.valueOf(typeMap.get("code"));
            String typeName = String.valueOf(typeMap.get("name"));
            if (!Arrays.stream(OrgCenterTypeEnum.values()).map(OrgCenterTypeEnum::getTypeCode).collect(Collectors.toList()).contains(typeCode)) {
                throw ExceptionResultEnum.ERROR.exception("机构类型在枚举中不存在 + typeCode = " + typeCode);
            }
            if (!OrgCenterTypeEnum.PARTNER.getTypeCode().equals(typeCode)) {
                BasicSchool school = new BasicSchool();
                school.setId(id);
                school.setCode(code);
                school.setName(name);
                school.setEnable(true);
                school.setAccessKey(accessKey);
                school.setAccessSecret(accessSecret);
                school.setLogo(logo);
                basicSchoolService.save(school);
            }
        }
    }
}
